# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.conf import settings
from django.core.cache import cache
from django.core import exceptions
from django.utils.translation import ugettext_lazy, ugettext_noop

import requests
from rest_framework import serializers
from shop.conf import app_settings
from shop.models.delivery import DeliveryItemModel
from shop.serializers.bases import BaseOrderItemSerializer


# status messages supplied by SendCloud via https://<apikey>:<apisecret>@panel.sendcloud.sc/api/v2/parcels/statuses
ugettext_noop("Announced")  # 1
ugettext_noop("Announced: not collected")  # 13
ugettext_noop("Announcement failed")  # 1002
ugettext_noop("message': 'En route to sorting center")  # 3
ugettext_noop("Delivery delayed")  # 4
ugettext_noop("Sorted")  # 5
ugettext_noop("Not sorted")  # 6
ugettext_noop("Being sorted")  # 7
ugettext_noop("Delivery attempt failed")  # 8
ugettext_noop("Delivered")  # 11
ugettext_noop("Awaiting customer pickup")  # 12
ugettext_noop("Error collecting")  # 15
ugettext_noop("Picked up from postoffice")  # 22
ugettext_noop("Unable to deliver")  # 80
ugettext_noop("Parcel en route")  # 91
ugettext_noop("Driver en route")  # 92
ugettext_noop("Picked up at service point")  # 93
ugettext_noop("No label")  # 999
ugettext_noop("Ready to send")  # 1000
ugettext_noop("Being announced")  # 1001
ugettext_noop("Cancelled")  # 2000
ugettext_noop("Submitting cancellation request")  # 2001
ugettext_noop("Cancellation requested")  # 1999
ugettext_noop("Unknown status - check carrier track & trace page for more insights")  # 1337


class OrderItemListSerializer(serializers.ListSerializer):
    def to_representation(self, data):
        try:
            queryset = data.order_by('deliveryitem__delivery')
        except exceptions.FieldError:
            queryset = data.all()
        return super(OrderItemListSerializer, self).to_representation(queryset)


class OrderItemSerializer(BaseOrderItemSerializer):
    parcel_url = 'https://panel.sendcloud.sc/api/v2/parcels/{parcel_id}'
    credentials = settings.SHOP_SENDCLOUD['API_KEY'], settings.SHOP_SENDCLOUD['API_SECRET']

    summary = serializers.SerializerMethodField(
        help_text="Sub-serializer for fields to be shown in the product's summary.")

    delivery_status = serializers.SerializerMethodField()
    parcel = serializers.SerializerMethodField()

    class Meta(BaseOrderItemSerializer.Meta):
        fields = ['line_total', 'unit_price', 'product_code', 'quantity', 'summary',
                  'delivery_status', 'parcel', 'extra']
        list_serializer_class = OrderItemListSerializer

    def get_summary(self, order_item):
        label = self.context.get('render_label', 'order')
        serializer_class = app_settings.PRODUCT_SUMMARY_SERIALIZER
        serializer = serializer_class(order_item.product, context=self.context,
                                      read_only=True, label=label)
        return serializer.data

    def get_delivery_status(self, order_item):
        parcel = self.get_parcel(order_item)
        if parcel:
            return ugettext_lazy(parcel['status']['message'])
        return ugettext_lazy("Purchase order issued")

    def get_parcel(self, order_item):
        try:
            delivery = order_item.deliveryitem_set.get().delivery
        except DeliveryItemModel.DoesNotExist:
            return
        parcel_url = self.parcel_url.format(parcel_id=delivery.shipping_id)
        parcel = cache.get(parcel_url)
        if parcel is None:
            response = requests.get(parcel_url, auth=self.credentials)
            if response.status_code >= 200 and response.status_code <= 299:
                parcel = response.json()['parcel']
                cache.set(parcel_url, parcel, 3600)  # TODO: make this configurable
        return parcel



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.template.loader import get_template
from django.utils.translation import ugettext_lazy as _
from phonenumber_field.modelfields import PhoneNumberField
from shop.models.customer import BaseCustomer


class Customer(BaseCustomer):
    SALUTATION = [('mrs', _("Mrs.")), ('mr', _("Mr.")), ('na', _("(n/a)"))]

    number = models.PositiveIntegerField(
        _("Customer Number"),
        null=True,
        default=None,
        unique=True,
    )

    salutation = models.CharField(
        _("Salutation"),
        max_length=5,
        choices=SALUTATION,
    )

    phone_number = PhoneNumberField(
        _("Phone number"),
        blank=True,
        null=True,
    )

    def get_number(self):
        return self.number

    def get_or_assign_number(self):
        if self.number is None:
            aggr = Customer.objects.filter(number__isnull=False).aggregate(models.Max('number'))
            self.number = (aggr['number__max'] or 0) + 1
            self.save()
        return self.get_number()

    @classmethod
    def reorder_form_fields(self, field_order):
        field_order.insert(0, 'salutation')
        field_order.append('phone_number')
        return field_order

    def as_text(self):
        template = get_template('shop_sendcloud/customer.txt')
        return template.render({'customer': self})



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import requests

from django.conf import settings
from django.core.management.base import BaseCommand, CommandError
from django.utils.translation import ugettext_lazy as _

from shop.money import MoneyMaker
from shop_sendcloud.models import ShippingMethod, ShippingDestination

EUR = MoneyMaker('EUR')


class Command(BaseCommand):
    help = _("Fetch shipping method fees from SendCloud.")
    download_url = 'https://panel.sendcloud.sc/api/v2/shipping_methods'
    credentials = settings.SHOP_SENDCLOUD['API_KEY'], settings.SHOP_SENDCLOUD['API_SECRET']
    INCLUDE_CARRIERES = settings.SHOP_SENDCLOUD.get('INCLUDE_CARRIERES')
    EXCLUDE_CARRIERES = settings.SHOP_SENDCLOUD.get('EXCLUDE_CARRIERES', ['sendcloud'])

    def handle(self, verbosity, *args, **options):
        response = requests.get(self.download_url, auth=self.credentials)
        if response.status_code != 200:
            msg = "Failed to import carriers from SendCloud. Reason: {message}"
            raise CommandError(msg.format(**response.json()['error']))
        response_data = response.json()
        for sm in response_data['shipping_methods']:
            if isinstance(self.INCLUDE_CARRIERES, (list, tuple)):
                if sm['carrier'] not in self.INCLUDE_CARRIERES:
                    continue
            elif isinstance(self.EXCLUDE_CARRIERES, (list, tuple)):
                if sm['carrier'] in self.EXCLUDE_CARRIERES:
                    continue

            id = sm.pop('id')
            default_price = sm.pop('price')
            sm.pop('service_point_input', None)
            countries = sm.pop('countries', [])
            try:
                shipping_method, created = ShippingMethod.objects.update_or_create(id=id, defaults=sm)
                if created:
                    destination_ids = []
                else:
                    destination_ids = list(shipping_method.destinations.values_list('id', flat=True))
            except Exception as ex:
                raise CommandError("In id={}: {}".format(id, ex))
            for dst in countries:
                country = dst.pop('iso_2')
                iso_3 = dst.pop('iso_3')
                dst.pop('id', None)
                dst.pop('name', None)
                dst.setdefault('price', default_price)
                try:
                    destination, created = ShippingDestination.objects.update_or_create(
                        shipping_method=shipping_method, country=country, defaults=dst)
                    if not created and destination.id in destination_ids:
                        destination_ids.remove(destination.id)
                except Exception as ex:
                    raise CommandError("In shipping_id={} country={}: {}".format(shipping_method.id, iso_3, ex))
            # remove destinations which haven't been updated
            shipping_method.destinations.filter(id__in=destination_ids).delete()



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

import requests
from django.conf import settings
from django.conf.urls import url
from django.contrib import messages
from django.http import HttpResponse
from django.utils import timezone
from django.utils.six.moves.urllib.parse import urlparse
from shop.models.delivery import DeliveryModel


class SendCloudOrderAdminMixin(object):
    change_form_template = 'shop_sendcloud/order_change_form.html'
    parcel_label_url = 'https://panel.sendcloud.sc/api/v2/labels/{}'
    credentials = settings.SHOP_SENDCLOUD['API_KEY'], settings.SHOP_SENDCLOUD['API_SECRET']

    class Media:
        js = ['shop/admin/js/order_change_form.js']
        css = {
            'all': ['shop/admin/css/order_change_form.css']
        }

    def render_change_form(self, request, context, add=False, change=False, form_url='', obj=None):
        assert obj is not None
        if obj.status == 'ready_for_delivery':
            context.setdefault('parcel_label_urls', [])
            try:
                for delivery in obj.delivery_set.filter(shipping_id__isnull=False, shipped_at__isnull=True):
                    url = self.parcel_label_url.format(delivery.shipping_id)
                    response = requests.get(url, auth=self.credentials)
                    if response.status_code == 200:
                        parcel = response.json()
                        context['parcel_label_urls'].append(parcel['label']['normal_printer'][2])  # TODO: make this configurable
            except:
                messages.add_message(request, messages.INFO, "No SendCloud label could be printed.")
        return super(SendCloudOrderAdminMixin, self).render_change_form(request, context, add, change, form_url, obj)

    def get_urls(self):
        my_urls = [
            url(r'^print_shipping_label/$', self.admin_site.admin_view(self.passthrough_shipping_label),
                name='print_shipping_label'),
        ]
        my_urls.extend(super(SendCloudOrderAdminMixin, self).get_urls())
        return my_urls

    def passthrough_shipping_label(self, request):
        label_url = request.GET.get('url')
        res = requests.get(label_url, auth=self.credentials)
        if res.status_code == 200:
            # mark the parcel label as printed
            try:
                parcel_id = urlparse(label_url).path.split('/')[-1]
                delivery = DeliveryModel.objects.get(shipping_id=parcel_id)
            except DeliveryModel.DoesNotExist:
                parcel_id = None
            else:
                delivery.shipped_at = timezone.now()
                delivery.save(update_fields=['shipped_at'])
        else:
            parcel_id = None
        response = HttpResponse(res.content, status=res.status_code, content_type=res.headers['content-type'])
        if parcel_id:
            response['Content-Disposition'] = 'filename="parcel_label_{}.pdf"'.format(parcel_id)
        return response



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.template.loader import get_template
from django.utils.translation import ugettext_lazy as _
from shop.models.address import BaseShippingAddress, BaseBillingAddress, CountryField


class AddressModelMixin(models.Model):
    """
    SendCloud prescribes an address model, which differs slightly from django-SHOP's default address model.
    Add this model to the merchant's ``model.py``, if ``shop_sendcloud`` is used as your delivery provider.
    """
    name = models.CharField(
        _("Full name"),
        max_length=1024,
    )

    company_name = models.CharField(
        _("Company name"),
        max_length=1024,
        blank=True,
        null=True,
    )

    address = models.CharField(
        _("Address line"),
        max_length=1024,
    )

    house_number = models.CharField(
        _("House number"),
        max_length=12,
    )

    postal_code = models.CharField(
        _("ZIP / Postal code"),
        max_length=12,
    )

    city = models.CharField(
        _("City"),
        max_length=1024,
    )

    country = CountryField(_("Country"))

    class Meta:
        abstract = True


class ShippingAddress(BaseShippingAddress, AddressModelMixin):
    class Meta:
        verbose_name = _("Shipping Address")
        verbose_name_plural = _("Shipping Addresses")

    def as_text(self):
        template = get_template('shop_sendcloud/address.txt')
        return template.render({'address': self})


class BillingAddress(BaseBillingAddress, AddressModelMixin):
    class Meta:
        verbose_name = _("Billing Address")
        verbose_name_plural = _("Billing Addresses")

    def as_text(self):
        template = get_template('shop_sendcloud/address.txt')
        return template.render({'address': self})



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from django.db import models
from django.utils.translation import ugettext_lazy as _
from shop.money.fields import MoneyField
from shop.models.customer import BaseCustomer


class ShippingMethod(models.Model):
    id = models.PositiveIntegerField(primary_key=True)

    name = models.CharField(
        max_length=255,
    )

    carrier = models.CharField(
        max_length=32,
    )

    min_weight = models.DecimalField(
        max_digits=6,
        decimal_places=3,
    )

    max_weight = models.DecimalField(
        max_digits=8,
        decimal_places=3,
    )

    updated_at = models.DateTimeField(
        _("Updated at"),
        auto_now=True,
    )

    class Meta:
        verbose_name = _("Shipping Method")
        verbose_name_plural = _("Shipping Methods")


class ShippingDestination(models.Model):
    shipping_method = models.ForeignKey(
        ShippingMethod,
        related_name='destinations',
    )

    country = models.CharField(max_length=3)

    price = MoneyField(currency='EUR')

    class Meta:
        verbose_name = _("Shipping Destination")
        verbose_name_plural = _("Shipping Destination")
        unique_together = ['country', 'shipping_method']



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from decimal import Decimal
import math
import requests
from django.conf import settings
from django.core.exceptions import ImproperlyConfigured, ValidationError
from django.db.utils import OperationalError, ProgrammingError
from django.utils.translation import ugettext_lazy as _
from shop.modifiers.pool import cart_modifiers_pool
from shop.serializers.cart import ExtraCartRow
from shop.shipping.modifiers import ShippingModifier
from shop.money import MoneyMaker
from shop_sendcloud.models import ShippingMethod, ShippingDestination

EUR = MoneyMaker('EUR')  # at SendCloud, everything is charged in Euros


class SendcloudShippingModifierBase(ShippingModifier):
    carrier = None
    create_parcel_url = 'https://panel.sendcloud.sc/api/v2/parcels/'
    cancel_parcel_url = 'https://panel.sendcloud.sc/api/v2/parcels/{}/cancel'
    credentials = settings.SHOP_SENDCLOUD['API_KEY'], settings.SHOP_SENDCLOUD['API_SECRET']
    error_message = _("Parcel can not be delivered: {}")

    def get_queryset(self):
        return ShippingMethod.objects.filter(carrier=self.carrier)

    def get_choice(self):
        qs = self.get_queryset().order_by('id')
        return (self.identifier, qs.first().name)

    def add_extra_cart_row(self, cart, request):
        if not self.is_active(cart) and len(cart_modifiers_pool.get_shipping_modifiers()) > 1:
            return

        weight = max(cart.weight, Decimal('0.001')).quantize(Decimal('0.000'))
        cheapest_destination = ShippingDestination.objects.filter(
            shipping_method__carrier=self.carrier,
            country=cart.shipping_address.country if cart.shipping_address else None,
            shipping_method__min_weight__lte=weight,
            shipping_method__max_weight__gte=weight,
        ).order_by('price').first()
        if cheapest_destination:
            parcel = self.get_sendcloud_parcel(cart)
            parcel.update(
                shipment={'id': cheapest_destination.shipping_method_id},
                weight=str(weight),
            )
            cart.extra['sendcloud_data'] = {'parcel': parcel}
            amount = EUR(cheapest_destination.price)
            instance = {'label': _("Shipping costs"), 'amount': amount}
            cart.extra_rows[self.identifier] = ExtraCartRow(instance)
            cart.total += amount
        else:
            instance = {'label': _("Unable to ship"), 'amount': EUR()}
            cart.extra_rows[self.identifier] = ExtraCartRow(instance)

    def get_sendcloud_parcel(self, cart):
        shipping_address = cart.shipping_address or cart.billing_address
        data = {
            'name': shipping_address.name,
            'address': shipping_address.address,
            'house_number': shipping_address.house_number,
            'city': shipping_address.city,
            'postal_code': shipping_address.postal_code,
            'email': cart.customer.email,
            'country': shipping_address.country,
            'insured_value': math.ceil(cart.subtotal / 100) * 100,
        }
        if shipping_address.company_name:
            data['company_name'] = shipping_address.company_name
        if cart.customer.phone_number:
            data['telephone'] = str(cart.customer.phone_number)
        return data

    def ship_the_goods(self, delivery):
        if not delivery.order.associate_with_delivery:
            msg = "Either add 'shop.shipping.workflows.PartialDeliveryWorkflowMixin' or \n" \
                  "  'shop.shipping.workflows.CommissionGoodsWorkflowMixin' to settings.SHOP_ORDER_WORKFLOWS"
            raise ImproperlyConfigured(msg)
        carrier = delivery.order.extra['shipping_modifier'][len('sendcloud:'):]
        destination_country = delivery.order.extra['sendcloud_data']['parcel']['country']
        if delivery.order.associate_with_delivery_items:
            delivery_items = delivery.deliveryitem_set.all()
            weight = sum([di.item.quantity * Decimal(di.item.product.get_weight()) for di in delivery_items])
            weight = max(weight, Decimal('0.001')).quantize(Decimal('0.000'))
        else:
            weight = delivery.order.extra['sendcloud_data']['parcel']['weight']
        insured_value = sum([di.item.line_total for di in delivery_items])
        cheapest_destination = ShippingDestination.objects.filter(
            shipping_method__carrier=carrier,
            country=destination_country,
            shipping_method__min_weight__lte=weight,
            shipping_method__max_weight__gte=weight,
        ).order_by('price').first()
        if not cheapest_destination:
            msg = "No shipping destination for carrier: {carrier}"
            raise ValidationError(msg.format(carrier=carrier))
        parcel = dict(
            delivery.order.extra['sendcloud_data']['parcel'],
            order_number=delivery.order.get_number(),
            request_label=True,
            weight=str(weight),
            shipment={'id': cheapest_destination.shipping_method_id},
            insured_value=math.ceil(insured_value / 100) * 100,
        )
        try:
            response = requests.post(self.create_parcel_url, json={'parcel': parcel}, auth=self.credentials)
            response_data = response.json()
            if response.status_code == 200:
                if not response_data['parcel'].get('label'):
                    raise Exception(_("Missing shipping label in response data['parcel']"))
                delivery.shipping_id = response_data['parcel']['id']
            else:
                raise Exception(response_data['error']['message'])
        except Exception as exc:
            raise ValidationError(self.error_message.format(exc))

    def withdraw_delivery(self, delivery):
        cancel_parcel_url = self.cancel_parcel_url.format(delivery.shipping_id)
        response = requests.post(cancel_parcel_url, auth=self.credentials)
        if response.status_code >= 200 and response.status_code <= 299:
            delivery.delete()


class SendcloudShippingModifiers(list):
    """
    A list of possible shipping modifiers for this provider
    """
    def __init__(self):
        try:
            ShippingMethod.objects.exists()
        except (OperationalError, ProgrammingError):
            return  # in case the database table does not exist yet
        for carrier in ShippingMethod.objects.values_list('carrier', flat=True).distinct():
            name = 'Sendcloud{}Modifier'.format(carrier.title())
            attrs = {
                'carrier': carrier,
                'identifier': 'sendcloud:{}'.format(carrier),
            }
            self.append(type(str(name), (SendcloudShippingModifierBase,), attrs))



# -*- coding: utf-8 -*-
# Generated by Django 1.11.7 on 2017-11-09 16:57
from __future__ import unicode_literals

from django.db import migrations, models
import django.db.models.deletion
import shop.money.fields


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='ShippingDestination',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('country', models.CharField(max_length=3)),
                ('price', shop.money.fields.MoneyField()),
            ],
            options={
                'verbose_name': 'Shipping Destination',
                'verbose_name_plural': 'Shipping Destination',
            },
        ),
        migrations.CreateModel(
            name='ShippingMethod',
            fields=[
                ('id', models.PositiveIntegerField(primary_key=True, serialize=False)),
                ('name', models.CharField(max_length=255)),
                ('carrier', models.CharField(max_length=32)),
                ('min_weight', models.DecimalField(decimal_places=3, max_digits=6)),
                ('max_weight', models.DecimalField(decimal_places=3, max_digits=8)),
                ('updated_at', models.DateTimeField(auto_now=True, verbose_name='Updated at')),
            ],
            options={
                'verbose_name': 'Shipping Method',
                'verbose_name_plural': 'Shipping Methods',
            },
        ),
        migrations.AddField(
            model_name='shippingdestination',
            name='shipping_method',
            field=models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, related_name='destinations', to='shop_sendcloud.ShippingMethod'),
        ),
        migrations.AlterUniqueTogether(
            name='shippingdestination',
            unique_together=set([('country', 'shipping_method')]),
        ),
    ]



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import unicode_literals
from setuptools import setup, find_packages
from shop_sendcloud import __version__

with open('README.md', 'r') as fh:
    long_description = fh.read()

CLASSIFIERS = [
    'Environment :: Web Environment',
    'Framework :: Django',
    'Intended Audience :: Developers',
    'License :: OSI Approved :: MIT License',
    'Operating System :: OS Independent',
    'Programming Language :: Python :: 2.7',
    'Programming Language :: Python :: 3.5',
    'Programming Language :: Python :: 3.6',
    'Topic :: Software Development :: Libraries :: Application Frameworks',
    'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
]

setup(
    author="Jacob Rief",
    author_email="jacob.rief@gmail.com",
    name='djangoshop-sendcloud',
    version=__version__,
    description='SendCloud Shipping Provider Integration for django-shop',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/jrief/djangoshop-sendcloud',
    license='MIT License',
    platforms=['OS Independent'],
    classifiers=CLASSIFIERS,
    install_requires=[
        'requests',
        'django-phonenumber-field',
        'phonenumbers',
    ],
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
)



# -*- coding: utf-8 -*-
from __future__ import unicode_literals

from shop.modifiers.pool import cart_modifiers_pool
from shop_sendcloud.modifiers import SendcloudShippingModifierBase


class SendclouldWorkflowMixin(object):
    def withdraw_from_delivery(self):
        for delivery in self.delivery_set.filter(shipping_id__isnull=False, shipping_method__startswith='sendcloud:'):
            shipping_modifier = cart_modifiers_pool.get_active_shipping_modifier(delivery.shipping_method)
            assert isinstance(shipping_modifier, SendcloudShippingModifierBase)
            shipping_modifier.withdraw_delivery(delivery)
